// Copyright 2016 Tadd Torborg -- I grant the right to modify and sell products using this sourcecode.  
//I insist you put my copyright notice and this line in every sourcecode file included in any project which uses my PWRMAN software. 
// If you do manage to construct or produce 2 or more devices based on this source code, please send an email to me at tadd@mac.com
//  Thanks!! 
// Source code by Tadd Torborg, ham-radio callsign KA2DEW, for PWRMAN power controller -- runs on TI MSP430F2012 CPU.
// Concept by Tadd Torborg.  Board design, layout, stuffed, and maintained by Stephen Snell, ham-radio callsign W4RFQ
// PWRMAN operation is documented on the TARPN web site.  See  http://tarpn.net/pwrman
// I compiled and debugged this project using IAR's EW430 version 5.
//

#define VERSIONNUMBER 4


/// IAR or TI Library includes
#include "string.h"
#include "stdlib.h"
#include "io430.h"
#include "intrinsics.h"

// PWRMAN includes
#include "common.h"
#include "sw_led.h"
#include "pwrman.h"
#include "pihellotoggle.h"

/// MSP430F2012 program for PWRMAN.
/// Hardware consists of 3 AtoD ports, one used to check the two switches, each of which produces a different voltage.
/// One is the battery and one is the main supply.  We only use the values on the two supply voltages as present/not present detect.
/// In version 2 we don't test to see if the battery is present at all.  There was a bug in the version 0 PCB which prohibited that test.
/// Also hardware has outputs for BLUE, RED and GREEN LEDs, Power control relay (actually a FET) to control if the Raspberry PI is powered or not,
/// two inputs from the Raspberry PI called power-up and hellotoggle (or blinky), and a GPIO output to tell the Rapberry PI to shutdown.

// P1 Digital signals         1=digital 0 = special function or not connected.
// xxxx xxxx
//         0.......P1.0/A0/TACLK - AtoD input A0 Mains volts sense divided by 7
//        0........P1.1/A1       - AtoD input A1 Button input resistor sense
//       0.........P1.2/A2       - AtoD input A2 Gel Cel volts sense divided by 7
//      1..........P1.3          - P1.3 output  Blue Blinky LED if low (driven through a transistor)
//    1............P1.4          - P1.4 output  RED switch LED if high
//   1.............P1.5          - P1.5 output  Raspberry PI command to shutdown if high
//  1..............P1.6          - P1.6 input  Raspberry PI is running, if high
// 1...............P1.7          - P1.7 input  Raspberry PI alive "hellotoggle" every half second

// P2 Digital signals         1=digital 0 = special function or not connected
// xxxx xxxx
//         0.......P2.0          - nc     These signals are not connected to the CPU pins
//        0........P2.1          - nc     These signals are not connected to the CPU pins
//       0.........P2.2          - nc     These signals are not connected to the CPU pins
//      0..........P2.3          - nc     These signals are not connected to the CPU pins
//    0............P2.4          - nc     These signals are not connected to the CPU pins
//   0.............P2.5          - nc     These signals are not connected to the CPU pins
//  1..............P2.6          - P2.6  output Power Output to the PI if high
// 1...............P2.7          - P2.7  output Green switch LED if high


/// This module contains the mainloop and the state machine.
///
/// The state machine keeps track of the mode we think the Raspberry PI is in and also the operations
/// requested by the user.
///
/// In addition there is a function and tables which manage blinking the BLUE status LED.  We send morse code to tell the user what state the PWRMAN is in.
/// There is not necessarily a direct correlation between the blinky messages and the powerManagerState.


/// A note on programming style:
/// Functions are capitalized and have CamelCase
/// Variables are not capitalized and have Camlecase.
/// All Variables end with _l _L or _G, or begin with "passed_".
/// _l is local to a function or sub-function
/// _L is global within a single source code module
/// _G is global to the project.  Only _G variables will show in include files.
/// passed_  is a variable passed from the caller of a function.
///
/// All globals are extern in include files.
///
/// None of the project's include files will include any other file.   (some of the TI or IAR includes may do so)
///
/// Functions may change _L and _G variables (yes, that sucks, but it saved stack space and CPU space).
///
/// All CAPS symbols are constants.  Macros (there are not any) names would be preceeded by "MACRO_"




#define TRUE  1
#define FALSE 0




//blinky messages
enum blinkyMessageType
{
   BLINKY_NONE,
   BLINKY_POWER_SAVE,
   BLINKY_IDLE_OFF,
   BLINKY_IDLE_RUNNING,
   BLINKY_DOWN_PI,
   BLINKY_UP_PI
};

struct blinky_STRUCT
{
   unsigned char          index;
   const unsigned char   *sequenceStart;
   unsigned char          divider;
   enum blinkyMessageType message;
} blinky_L;


unsigned short          pi_timerAfterCommsEnds_L;              //countdown for how long we wait after PI stops signalling before we yank the power
unsigned short          mainSupplyFailureTimer_L;              //While RUNNING, counts how long the main supply has been missing
unsigned short          mainPowerIntermittentBlinkCounter_L;   //This is the RED LED (button) blink cycle timer.  How many mainloop cycles before we turn off/on the LED. .
volatile unsigned char  lowPowerCpuModeNeeded_L;               //if true, then all of the outputs are in least-power condition except the blue blinky.   Never true if Raspberry PI is running.
unsigned char           humanWantsPiOnline_L;                  //This keeps track of what to do when main power comes back.  If we go from no main power to main power present, start up the Raspberry PI.
volatile unsigned short triggerLoopExecution_L;                //set by the timer interrupt to tell mainloop to run through one cycle.

static void InitTimerA(void);
static void MainLoopCycle(void);
static void UpdateBlueBlinkyPattern(enum blinkyMessageType code_p);


enum powerManagerStateType
{
   STATE_LOST_TOUCH_WITH_PI_WHILE_UP    = 0,
   STATE_IDLE_OFF_MAINOK                = 1,
   STATE_IDLE_OFF_MAINOFF               = 2,
   STATE_UP_PI                          = 3,
   STATE_RUNNING_MAIN_OK                = 4,
   STATE_RUNNING_MAIN_INTERMITTENT      = 5,
   STATE_DOWN_PI                        = 73,
   STATE_DOWN_PI_WAIT_AFTER_COMMS_ENDS  = 88
} powerManagerState_L;






///##################################################################
///##################################################################
///####                           _            __   __         ######
///####                          (_)          / /   \ \        ######
///####       _ __ ___     __ _   _   _ __   | |     | |       ######
///####      | '_ ` _ \   / _` | | | | '_ \  | |     | |       ######
///####      | | | | | | | (_| | | | | | | | | |     | |       ######
///####      |_| |_| |_|  \__,_| |_| |_| |_| | |     | |       ######
///####                                       \_\   /_/        ######
///####                                                        ######
///##################################################################
///##################################################################


void main( void )
{
   TurnOnPeripherals();
   WriteRelayControl(OFF);
   WritePiShutdown(FALSE);       //since relay is off, don't drive pi-shutdown.  it could hurt

   /// ########### ANNOUNCE VERSION NUMBER by blinking BLUE LED once per version count ########
   WriteBlueLED(OFF);                                                               ////#######
   WriteGreenLED(ON);        //turn on GREEN button                                 ////#######
   WriteRedLED(ON);          //turn on RED button                                   ////#######
   ShortDelay(34000);                                                               ////#######
   for(unsigned int blinkCount_l=0; blinkCount_l<VERSIONNUMBER; blinkCount_l++)     ////#######
   {                                                                                ////#######
      WriteBlueLED(ON);                                                             ////#######
      ShortDelay(1000);                                                             ////#######
      WriteBlueLED(OFF);                                                            ////#######
      ShortDelay(34000);                                                            ////#######
   }                                                                                ////#######
                                                                                    ////#######
   ShortDelay(34000);                                                               ////#######
   WriteGreenLED(OFF);        //turn on GREEN button                                ////#######
   WriteRedLED(OFF);          //turn on RED button                                  ////#######
   ShortDelay(34000);                                                               ////#######
                                                                                    ////#######
   /// ########### Done with Version anouncement. #############################################

   // We have just powered on.  Set up initial conditions.   The Raspberry PI is plugged into PWRMAN's output so it must be off now.

   lowPowerCpuModeNeeded_L             = FALSE;                        //if true, then all of the outputs are in least-power condition except the blue blinky.   Never true if Raspberry PI is running.
   humanWantsPiOnline_L                = FALSE;                        //This keeps track of what to do when power is restored.  If we go from no main power to main power present, start up the Raspberry PI.
   mainPowerIntermittentBlinkCounter_L = 0;                            //This is the RED LED (button) blink cycle timer.  How many mainloop cycles before we turn off/on the LED. .

   blinky_L.message                    = BLINKY_NONE;                  //blinky is the BLUE LED.  We send blink sequences to clue the operator as to what mode the PWRMAN is in.    NONE means no code.  It can also mean don't change.
   UpdateBlueBlinkyPattern(BLINKY_IDLE_OFF);                           //This says to start flashing the idle blink, which is one short flash every second.

   powerManagerState_L                 = STATE_IDLE_OFF_MAINOK;         //Select this as a startup state because it doesn't assert "supposed to be asleep".  This also agrees with the IDLE-OFF flash code

   //PI HELLO TOGGLE is a binary square wave signal from the raspberrY PI to indicate that it is running its PWRMAN responder program.
   InitPiHelloToggle();

   // Stop watchdog timer to prevent time out reset
   WDTCTL = WDTPW + WDTHOLD;
   InitTimerA();

   while(1)
   {
      __bis_SR_register(GIE);
      while(triggerLoopExecution_L == 0)     //triggerLoopExecution is triggered by the timer interrupt to tell mainloop to run through one cycle.
      { }                                    //not triggered?  Hang out until we are.

      triggerLoopExecution_L = 0;            //set the trigger for next time
      WDTCTL = WATCHDOG_WORD;                // reset watchdog timer


      // supposed-to-be-asleep is only set to TRUE in the power-off state.
      if(lowPowerCpuModeNeeded_L == TRUE)
      {
         TurnOnPeripherals();                //restore peripherals so we can check the power supply and buttons.
      }


      MainLoopCycle();                       //Here is where the meat of the PWRMAN program is.



      if(lowPowerCpuModeNeeded_L == TRUE)
      {
         TurnOffPeripheralsForLowPowerMode();    //if we're supposed to be asleep, turn off all of the peripherals we can afford to and set i/os to least power mode, except for the blue light.
         __low_power_mode_1();
      }
   }
}






//###############################################################################
//######   _______   _                                                    #######
//######  |__   __| (_)                                        /\         #######
//######     | |     _   _ __ ___     ___   _ __   ______     /  \        #######
//######     | |    | | | '_ ` _ \   / _ \ | '__| |______|   / /\ \       #######
//######     | |    | | | | | | | | |  __/ | |              / ____ \      #######
//######     |_|    |_| |_| |_| |_|  \___| |_|             /_/    \_\     #######
//######                                                                  #######
//###############################################################################


//////// Timer A is used to trigger a run through the main-loop-cycle.


//##################################################################
// INIT TIMER A
//
// Called at boot time to initialize the mainloop timer.
static void InitTimerA(void)
{
  CCTL0 = CCIE;                             // CCR0 interrupt enabled
  TACTL = TASSEL_2 + MC_1 + ID_3;           // SMCLK/8, upmode
  CCR0 =  10000;                            // Something around 1/12th of a second.
}


//##################################################################
// Called each timer cycle.  This function forces the CPU into full awake mode and sets a flag used to
// tell mainloop that a timer cycle has occured.
#pragma vector=TIMERA0_VECTOR      // 0xFFF2 Timer A CC0
__interrupt void TIMERA0____Interrupt(void)
{
   triggerLoopExecution_L++;
   __low_power_mode_off_on_exit();                                                           // come back up in full power.
}








////################################################################################################################
////################################################################################################################
////####                                                                                                     #######
////####  __  __           _           _                                 _____                  _            #######
////#### |  \/  |         (_)         | |                               / ____|                | |           #######
////#### | \  / |   __ _   _   _ __   | |        ___     ___    _ __   | |       _   _    ___  | |   ___     #######
////#### | |\/| |  / _` | | | | '_ \  | |       / _ \   / _ \  | '_ \  | |      | | | |  / __| | |  / _ \    #######
////#### | |  | | | (_| | | | | | | | | |____  | (_) | | (_) | | |_) | | |____  | |_| | | (__  | | |  __/    #######
////#### |_|  |_|  \__,_| |_| |_| |_| |______|  \___/   \___/  | .__/   \_____|  \__, |  \___| |_|  \___|    #######
////####                                                       | |                __/ |                      #######
////####                                                       |_|               |___/                       #######
////################################################################################################################
////################################################################################################################
//
// Called from main() once every timer cycle.
// This function checks buttons and power supply, runs timers, lights the GREEN and RED LEDs, blinks the BLUE LED.

/// This constant declares how many mainloop cycles we will wait after main power
/// goes away before shutting down the Raspberry PI.  2000 is just under 3 minutes.
#define TIMEOUT_FOR_POWER_FAIL 2000

unsigned short calculatedBlinkTime_L;     //temperary variable used to calculate the blink rate during main-powersupply-loss

static void MainLoopCycle(void)
{
   UpdateBlueBlinkyPattern(BLINKY_NONE);
   UpdatePiHelloToggle();

   WriteGreenLED(OFF);        //turn off GREEN button
   WriteRedLED(OFF);          //turn off RED button
   WritePiShutdown(FALSE);    //don't drive the PI wire.  It could be bad
   lowPowerCpuModeNeeded_L = FALSE;
   pi_timerAfterCommsEnds_L++;                             //this counter is used to blink LEDs and to keep track of how long the Raspberry PI has not been communicative.
   switch(powerManagerState_L)
   {
                                                           ///############# LOST TOUCH WITH PI WHILE UP
                                                           /// We get this state from fully-up if the Raspberry PI is removed from power or the ribbon cable is detached.
                                                           /// Stay here until the Raspberry PI answers again, or the main power goes away, or the RED button is pressed.

      case STATE_LOST_TOUCH_WITH_PI_WHILE_UP:              //assume the Raspberry PI is supposed to be powered up
         WriteRelayControl(TRUE);                          //turn on relay
         UpdateBlueBlinkyPattern(BLINKY_IDLE_OFF);         //blink idle
         if(0 != (pi_timerAfterCommsEnds_L & 0x2) )
         {
            WriteGreenLED(ON);                             //panic flash both RED and GREEN LEDs
            WriteRedLED(ON);                               //panic flash both RED and GREEN LEDs
         }

                                                           //Now check if we are back in touch with the Raspberry PI
         if(piHelloToggle_GoodTransitionsInARow_G >= REQUIRED_PI_HELLO_GOOD_TRANSITIONS_FOR_CONFIDENCE)
         {
            powerManagerState_L = STATE_RUNNING_MAIN_OK;
         }
         else if(ReadPushButtons() == RED_PUSHED)               //user can take us out of this LOST TOUCH state and turn off the system
         {
            humanWantsPiOnline_L = FALSE;                       //since the user is putting is down, don't come back up when power comes back.
            powerManagerState_L  = STATE_DOWN_PI;               // Next state if user presses RED button is "Down" state.  Will light RED button and signal "D" _..
         }
         else if(ReadMainSupplyAlive() == FALSE)                //if we lost touch and main supply also goes away, go to main off -- sleep
         {
            powerManagerState_L = STATE_IDLE_OFF_MAINOFF;
         }

         break;

                                                            ///############# IDLE-OFF-MAIN-OK
                                                            /// In this state we have the relay open so the Raspberry PI is not powered,
                                                            /// but we have a main power supply.  We don't need to be in low power mode.
                                                            /// We're not supposed to be in this state of the humanWantsPiOnline_L flag is set.
                                                            /// In that case, bring up the PI.  Also bring it up if the user presses GREEN button.
      case STATE_IDLE_OFF_MAINOK:
         WriteRelayControl(OFF);                            //turn off relay
         UpdateBlueBlinkyPattern(BLINKY_IDLE_OFF);
         if(ReadMainSupplyAlive() == FALSE)
         {
            powerManagerState_L = STATE_IDLE_OFF_MAINOFF;
         }
         else if((ReadPushButtons() == GREEN_PUSHED)  || (humanWantsPiOnline_L == TRUE))
         {
            powerManagerState_L = STATE_UP_PI;
            humanWantsPiOnline_L = TRUE;
         }
         break;
                                                            //############## IDLE-OFF-MAIN-OFF
                                                            //THe main power supply is gone and the Raspberry PI is already shut down.
                                                            // In this mode we blink the blue light every 5 seconds or so, and we
                                                            // don't allow anything to happen that uses power.  We do check and
                                                            // see if the human presses the green button.  If they do, we complain
                                                            // by lighting the RED light.
                                                            // We are asserting that we are supposed-to-be-asleep so every cycle through
                                                            // mainloop the CPU is put into a low power shut-down mode.
      case STATE_IDLE_OFF_MAINOFF:
         WriteRelayControl(OFF);                            //turn off relay
         UpdateBlueBlinkyPattern(BLINKY_POWER_SAVE);
         if(ReadMainSupplyAlive() == TRUE)
         {
            powerManagerState_L = STATE_IDLE_OFF_MAINOK;
         }
         else if(ReadPushButtons() == GREEN_PUSHED)
         {
            WriteRedLED(ON);                                //put on RED light while the GREEN button is pushed.  This tells the user there is a problem
         }
         else
         {
            lowPowerCpuModeNeeded_L = TRUE;                 //only go to low power shutdown if Main supply is disconnected and nobody disturbs us.
         }
         break;


                                                            //############## UP-PI
                                                            // Start the Raspberry PI
                                                            // The Raspberry PI is being told to start but we haven't heard from iet yet.
                                                            // Once we get good transitions on the hello wire from the Raspberry PI we
                                                            // will move to RUNNING.
      case STATE_UP_PI:
         WriteGreenLED(ON);         //turn on GREEN button
         WriteRelayControl(ON);     //turn on relay
         UpdateBlueBlinkyPattern(BLINKY_UP_PI);
         if(piHelloToggle_GoodTransitionsInARow_G >= REQUIRED_PI_HELLO_GOOD_TRANSITIONS_FOR_CONFIDENCE)
         {
            powerManagerState_L = STATE_RUNNING_MAIN_OK;
         }
         break;

                                                            //############## RUNNING, MAIN INTERMITTENT
                                                            // Main power supply went out while we were running!
                                                            // Stay running for a couple of minutes.  If main doesn't recover, tell PI to shut down.
                                                            // In the mean time, flash the RED switch faster and faster until we time out.
                                                            // If the RED button is pressed, go to DOWN-PI to tell the PI to go down immediately.
                                                            // If the power supply comes back, go to RUNNING, MAIN OK.
     case STATE_RUNNING_MAIN_INTERMITTENT:
         mainSupplyFailureTimer_L++;
         WriteRelayControl(ON);       //turn on relay
         UpdateBlueBlinkyPattern(BLINKY_IDLE_RUNNING);

         calculatedBlinkTime_L = 2+((TIMEOUT_FOR_POWER_FAIL - mainSupplyFailureTimer_L)/64);
         mainPowerIntermittentBlinkCounter_L++;
         if(mainPowerIntermittentBlinkCounter_L > calculatedBlinkTime_L)
         {
            mainPowerIntermittentBlinkCounter_L = 0;
            WriteRedLED(ON);        //turn on RED button
         }


         if(piHelloToggle_GoodTransitionsInARow_G < REQUIRED_PI_HELLO_GOOD_TRANSITIONS_FOR_CONFIDENCE)
         {
            powerManagerState_L      = STATE_LOST_TOUCH_WITH_PI_WHILE_UP;                             //hmm... no longer getting "hello" from the PI.
            pi_timerAfterCommsEnds_L = 0;                                                             //This could lead to an immediate power removal.
         }
         else if(ReadPushButtons() == RED_PUSHED)
         {
            powerManagerState_L  = STATE_DOWN_PI;
            humanWantsPiOnline_L = FALSE;
         }
         else if(ReadMainSupplyAlive() == TRUE)
         {
            powerManagerState_L = STATE_RUNNING_MAIN_OK;
         }
         else if(mainSupplyFailureTimer_L > TIMEOUT_FOR_POWER_FAIL)
         {
            powerManagerState_L = STATE_DOWN_PI;
         }
         break;

                                                            //############## RUNNING, MAIN OK
                                                            //Raspberry PI is running Linux and telling us that it is ok.  Main supply is on.
                                                            // We stay here until some thing happens.  RED button will tell us to shut down.
      case STATE_RUNNING_MAIN_OK:
         WriteRelayControl(ON);       //turn on relay
         UpdateBlueBlinkyPattern(BLINKY_IDLE_RUNNING);
         if(piHelloToggle_GoodTransitionsInARow_G < REQUIRED_PI_HELLO_GOOD_TRANSITIONS_FOR_CONFIDENCE)
         {
            powerManagerState_L      = STATE_LOST_TOUCH_WITH_PI_WHILE_UP;
            pi_timerAfterCommsEnds_L = 0;
         }
         else if(ReadPushButtons() == RED_PUSHED)
         {
            powerManagerState_L = STATE_DOWN_PI;
            humanWantsPiOnline_L = FALSE;
         }
         else if(ReadMainSupplyAlive() == FALSE)
         {
            mainSupplyFailureTimer_L = 0;
            mainPowerIntermittentBlinkCounter_L = 99;
            powerManagerState_L = STATE_RUNNING_MAIN_INTERMITTENT;
         }
         break;


                                                            //############## DOWN, PI
                                                            // This state tells the Raspberry PI to shutdown.  The message is
                                                            // given continuously until the Raspberry PI de-asserts it's I'm Awake GPIO24.
      case STATE_DOWN_PI:
         WriteRedLED(ON);         //turn on RED button
         WriteRelayControl(ON);   //turn on relay
         WritePiShutdown(TRUE);   //shut down the raspberry PI

         UpdateBlueBlinkyPattern(BLINKY_DOWN_PI);
         if(ReadPiPoweredUp() == 0)
         {
            pi_timerAfterCommsEnds_L = 0;
            powerManagerState_L = STATE_DOWN_PI_WAIT_AFTER_COMMS_ENDS;
         }
         break;


                                                            ///############## DOWN, PI -- WAIT AFTER COMMS ENDS
                                                            /// We're bringing down the PI but we only recently saw the PI drop it's GPIO24.
                                                            /// Wait in this state until we've gone 25 cycles (2 seconds) after the
                                                            /// last time we heard from the Raspberry PI.  This way we can be sure it
                                                            /// has fully shut down.
      case STATE_DOWN_PI_WAIT_AFTER_COMMS_ENDS:
         if(0 != (pi_timerAfterCommsEnds_L & 0x2) )
         {
            WriteRedLED(ON);        //blink on RED button
         }
         WriteRelayControl(TRUE);   //Relay stays on for a while
         WritePiShutdown(FALSE);    //Stop asserting the go-to-sleep
         UpdateBlueBlinkyPattern(BLINKY_DOWN_PI);
         if(pi_timerAfterCommsEnds_L > 25)
         {
            powerManagerState_L = STATE_IDLE_OFF_MAINOK;
         }
         break;





      default:
         /// we should never get here.  We don't have any way to complain about a firmware error so go to a safe state.
         powerManagerState_L      = STATE_LOST_TOUCH_WITH_PI_WHILE_UP;
         pi_timerAfterCommsEnds_L = 0;
         break;
   }

}




///######################################################################################################################
///######################################################################################################################
///####      ____    _                    ____    _   _           _                _        ______   _____        #######
///####     |  _ \  | |                  |  _ \  | | (_)         | |              | |      |  ____| |  __ \       #######
///####     | |_) | | |  _   _    ___    | |_) | | |  _   _ __   | | __  _   _    | |      | |__    | |  | |      #######
///####     |  _ <  | | | | | |  / _ \   |  _ <  | | | | | '_ \  | |/ / | | | |   | |      |  __|   | |  | |      #######
///####     | |_) | | | | |_| | |  __/   | |_) | | | | | | | | | |   <  | |_| |   | |____  | |____  | |__| |      #######
///####     |____/  |_|  \__,_|  \___|   |____/  |_| |_| |_| |_| |_|\_\  \__, |   |______| |______| |_____/       #######
///####                                                                   __/ |                                   #######
///####                                                                  |___/                                    #######
///####                                                                                                           #######
///######################################################################################################################
///######################################################################################################################



#define _ON  1
#define FAST 2
#define WRAP 0xFF
const unsigned char lowpowerBlink_Sequence_L[] = {FAST, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, WRAP };
const unsigned char off____Blinky_Sequence_L[] = {FAST, OFF, OFF, OFF, OFF, OFF, OFF, WRAP };
const unsigned char runningBlinky_Sequence_L[] = { OFF, OFF, _ON, OFF, OFF, OFF, _ON, _ON, _ON, _ON, OFF, OFF, OFF, _ON, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, WRAP };
const unsigned char go_up__Blinky_Sequence_L[] = { OFF, _ON, OFF, OFF, OFF, _ON, OFF, OFF, OFF, _ON, _ON, _ON, _ON, _ON, _ON, OFF, OFF, OFF, OFF, OFF, OFF, OFF, WRAP };
const unsigned char go_downBlinky_Sequence_L[] = { OFF, _ON, _ON, _ON, _ON, _ON, _ON, OFF, OFF, OFF, _ON, OFF, OFF, OFF, _ON, OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF, WRAP };


//###################################################
// UPDATE BLUE BLINKY PATTERN
//
// This function sends a single Morse character over and over on the blue LED so as to
// inform the user what the PWRMAN is trying to acheive at this time.  The codes are
// defined in an enum at the top of this file.
//
// If this function is called with BLINKY_NONE, then we continue running through the message
// we were already running.  Similarly, if it is called with the same message we were in,
// we continue running without restarting the message.
//
#define DIVIDER_START 2
static void UpdateBlueBlinkyPattern(enum blinkyMessageType passed_code)
{
   if((passed_code != BLINKY_NONE) && (passed_code != blinky_L.message))   //have we changed messages?
   {
      blinky_L.message = passed_code;        //Sequence has changed!  What's the new one?
      blinky_L.divider = DIVIDER_START;      //Divider slows down the code so we only have to read the sequence every other Main Loop Cycle.
      blinky_L.index = 0;
      switch(blinky_L.message)
      {
         case BLINKY_POWER_SAVE:   blinky_L.sequenceStart = &lowpowerBlink_Sequence_L[0];    break;
         case BLINKY_IDLE_OFF:     blinky_L.sequenceStart = &off____Blinky_Sequence_L[0];    break;
         case BLINKY_IDLE_RUNNING: blinky_L.sequenceStart = &runningBlinky_Sequence_L[0];    break;
         case BLINKY_DOWN_PI:      blinky_L.sequenceStart = &go_downBlinky_Sequence_L[0];    break;
         case BLINKY_UP_PI:        blinky_L.sequenceStart = &go_up__Blinky_Sequence_L[0];    break;
         case BLINKY_NONE:         blinky_L.sequenceStart = &off____Blinky_Sequence_L[0];    break;
         default:                  blinky_L.sequenceStart = &off____Blinky_Sequence_L[0];    break;
      }
   }

   if(blinky_L.divider != 0)
   {
      blinky_L.divider--;

   }
   else
   {
      unsigned char blinkOnOffResult_l;
      blinkOnOffResult_l = blinky_L.sequenceStart[blinky_L.index];

      blinky_L.divider = DIVIDER_START;

      if(blinkOnOffResult_l == WRAP)                                  //if the element is WRAP, then go back to the start of the sequence, and then reload the first value
      {
         blinky_L.index = 0;                                          //back to the start of the sequence
         blinkOnOffResult_l = blinky_L.sequenceStart[blinky_L.index]; //load the first element.. then continue processing
      }

      if(blinky_L.sequenceStart[blinky_L.index] == FAST)
      {
         blinkOnOffResult_l = ON;                            //fast means blink, and then move immediately to the next element
         blinky_L.divider = 1;
      }
      WriteBlueLED(blinkOnOffResult_l);
      blinky_L.index++;                                      //point to the next element of the message to send
   }
}









