// Copyright 2016 Tadd Torborg -- I grant the right to modify and sell products using this sourcecode.  
//I insist you put my copyright notice and this line in every sourcecode file included in any project which uses my PWRMAN software. 
// If you do manage to construct or produce 2 or more devices based on this source code, please send an email to me at tadd@mac.com
//  Thanks!! 
// Source code by Tadd Torborg, ham-radio callsign KA2DEW, for PWRMAN power controller -- runs on TI MSP430F2012 CPU.
// Concept by Tadd Torborg.  Board design, layout, stuffed, and maintained by Stephen Snell, ham-radio callsign W4RFQ
// PWRMAN operation is documented on the TARPN web site.  See  http://tarpn.net/pwrman
// I compiled and debugged this project using IAR's EW430 version 5.
//
/// IAR or TI Library includes
#include "string.h"
#include "stdlib.h"
#include "io430.h"


// PWRMAN includes
#include "common.h"
#include "pwrman.h"
#include "sw_led.h"


// P1DIR     CONFIG    0=input, 1=output.
// xxxx xxxx
//         0.......P1.0/A0/TACLK - AtoD input A0 Mains volts sense divided by 7
//        0........P1.1/A1       - AtoD input A1 Button input resistor sense
//       0.........P1.2/A2       - AtoD input A2 Gel Cel volts sense divided by 7
//      1..........P1.3          - P1.3 output  Blue Blinky LED if low (driven through a transistor)
//    1............P1.4          - P1.4 output  RED switch LED if high
//   1.............P1.5          - P1.5 output  Raspberry PI command to shutdown if high
//  0..............P1.6          - P1.6 input  Raspberry PI is running, if high
// 0...............P1.7          - P1.7 input  Raspberry PI alive "hellotoggle" every half second

// P2DIR     CONFIG    0=input, 1=output.
// xxxx xxxx
//         0.......P2.0          - nc
//        0........P2.1          - nc
//       0.........P2.2          - nc
//      0..........P2.3          - nc
//    0............P2.4          - nc
//   0.............P2.5          - nc
//  1..............P2.6          - P2.6 output Power Output to the PI if high
// 1...............P2.7          - P2.7 output Green switch LED if high



///########################################################################################
// SHORT DELAY
//
//
void ShortDelay(unsigned short passed_delayLength)
{
   unsigned short x_l;                                         // WDTCTL is declared volatile, so x does not need to be volatile too.
   for(x_l = 0; x_l < passed_delayLength; x_l++)
   {
      WDTCTL = WATCHDOG_WORD;                                  // reset watchdog timer
   }
}
//####################################################
// READ UNBUFFERED ATOD
// used for reading optical switches, internal temperature and internal CPU voltage,
// Reference must = REF2_5V or 0 (for 1.5V), channel specifies the analog input to read.
unsigned short ADC_ReadUnbufferedAtoDPort(unsigned short channel_p, unsigned short reference_p)
{
   unsigned short result_l;                                         // converted value to return

   ADC10CTL0 = 0;                                                   // reset ADC control registers
   ADC10CTL1 = 0;
   WDTCTL    = WDTPW + WDTHOLD;                                     // Stop WDT
   ADC10CTL1 = channel_p | ADC10DIV_4;                              // AVcc/2, 5MHz / 4
   ADC10CTL0 = SREF_1 + ADC10SHT_3 + REFON + ADC10ON + reference_p; // 64 clk samp: ~51 us samp.
   ShortDelay(40);                                                  // delay to let reference settle
   WDTCTL    = WDTPW + WDTHOLD;                                     // Stop WDT

   ADC10CTL0 |= ENC + ADC10SC;                                      // Sampling and conversion start
   while (ADC10CTL1 & ADC10BUSY)    ;                               // wait for conversion finished
   WDTCTL    =  WATCHDOG_WORD;                                      // reset and re-enable watchdog timer
   result_l  =  ADC10MEM;                                           // get conversion results
   ADC10CTL0 &= ~ENC;                                               // turn off the ENC bit to stop conversion
   ADC10CTL0 =  0;                                                  // now turn off the ADC10 enable and reference enables
   return(result_l);                                                // return conversion results
}



//################################################
// WRITE RELAY CONTROL
// This output drives the power on/off control which powers the Raspberry PI.
// Call with a 0 to turn off the Raspberry PI, or any other value to turn the Raspberry PI on.
//
void WriteRelayControl(unsigned char passed_turnOnPower)
{
   if(passed_turnOnPower)
   {
      P2OUT |= PORT26_BITMASK_PWRCONTROL;    //turn on RED switch LED by making the output low
   }
   else
   {
      P2OUT &= ~PORT26_BITMASK_PWRCONTROL;     //turn off RED switch LED by making the output high
   }
}



//################################################
// WRITE PI SHUTDOWN
//
// This function drives a GPIO which is read by GPIO11 on the Raspberry PI.
// If the Raspberry PI sees GPIO11 as a high, Linux calls "sudo shutdown -h now"
//
// call with a 0 to tell the Raspberry PI to keep running, or anything else to tell the Raspberry PI to shut-down Linux
void WritePiShutdown(unsigned char passed_shutdownNow)
{
   if(passed_shutdownNow)
   {
      P1OUT |= PORT15_BITMASK_PI_SHUTDOWN_COMMAND;    //turn on RED switch LED by making the output low
   }
   else
   {
      P1OUT &= ~PORT15_BITMASK_PI_SHUTDOWN_COMMAND;     //turn off RED switch LED by making the output high
   }
}



//################################################
// TURN ON PERIPHERALS
//
// This function is called when the MSP430 wakes from low-power-shutdown.
// During sleep these peripherals are placed in the least power consuming states.
// We only do low power shutdown when the Raspberry PI is off and main power is not available.

void TurnOnPeripherals(void)
{
   P1SEL = 0x7;                               // AtoD ports are Selected.
   P1DIR = 0x38;                              //Set AtoDs and all inputs to input
   P1OUT &= PORT13_BITMASK_BLUE_LED;          // drive GPIOs to off state -- leave blue the way it is
   P2SEL = 0;                                 // Set the two GPIO ports to outputs.
   P2DIR = 0xC0;                              //
   P2OUT = 0x00;                              // Drive the LED off and the relay to OFF
}



//#####################################################
// TURN OFF PERIPHERALS FOR LOW POWER MODE
//
// Called immediately before entering low power shutdown mode.
void TurnOffPeripheralsForLowPowerMode(void)
{
   while(ADC10CTL0) ADC10CTL0 = 0;                  // turn off ADC
   ADC10CTL1 = 0;
   ADC10AE0  = 0;
   ADC10AE1  = 0;

   // default the flash control registers
   FCTL1 = 0xA500;                                  // write the key and turn off all functions.

   P2IE  = 0;                                       // no interrupts configured port 2
   P2IES = 0;                                       // edge select to rising
   P1IE  = 0;                                       // no interrupts configured port 1
   P1IES = 0;                                       // edge select to rising



   // configure i/o for best current consumption.  We presume that the peripherals and specific i/o users will
   //  reconfigure these ports again the next time they are needed!
   // P1DIR RESET CONFIG    0=input, 1=output.
   // xxxx xxxx
   //         0.......P1.0 - AtoD input for Mains Volts Sense
   //        0........P1.1 - AtoD input for Pushbuttons
   //       0.........P1.2 - AtoD input for Gel Cel Volts Sense
   //      0..........P1.3 - GPIO Output for Blue Blinky  Active Low
   //    0............P1.4 - GPIO Output for Red LED  Active High
   //   0.............P1.5 - GPIO Output for Raspberry PI Shutdown command.  High means shutdown
   //  0..............P1.6 - GPIO Input for Raspberry PI Running.  High means running
   // 0...............P1.7 - GPIO Input for Raspberry PI HELLO toggle.
   P1SEL = 0;                                 // all ports are GPIO
   P1DIR = 0x38;                              //Set AtoDs and all inputs to input
   P1OUT &= PORT13_BITMASK_BLUE_LED;                 // drive GPIOs to off state -- leave blue the way it is
   P1REN = 0;                                 //  resistor enables off

   // P2DIR RESET CONFIG    0=input, 1=output.
   // xxxx xxxx
   //         0.......P2.0 - reserved
   //        0........P2.1 - reserved
   //       0.........P2.2 - reserved
   //      1..........P2.3 - reserved
   //    0............P2.4 - reserved
   //   0.............P2.5 - reserved
   //  0..............P2.6 - GPIO Output for Relay Control Output Active High
   // 0...............P2.7 - GPIO Output for Green LED Active high
   P2SEL = 0;                                 // Set the two GPIO ports to outputs.
   P2DIR = 0xC0;                              //
   P2OUT = 0x00;                              // Drive the LED off and the relay to OFF
   P2REN = 0;                                 // resistor enables off

}



///################################################
/// READ PI POWERED UP
///
/// The Raspberry PI outputs a high on GPIO24 when the power manager script starts (if the power manager script sees the ribbon cable connected!)
/// It leaves GPIO24 high indefinitely unless the ribbon cable is unplugged.
/// If powerdown is requested from the PWRMAN end, the script on the Raspberry PI leaves GPIO24 asserted as high.
/// Eventually, during shutdown, the Raspberry PI's CPU tristates the pin.
///
/// This function reads that GPIO from the PWRMAN end.
///
unsigned char ReadPiPoweredUp(void)
{
   if((P1IN & PORT16_BITMASK_PI_SAYS_IT_IS_UP) != 0)
   {
      return 1;
   }
   else
   {
      return 0;
   }
}


///################################################
/// READ PI HELLO TOGGLE
///
/// This looks at an input on the PWRMAN which is driven by the Raspberry PI's GPIO9.
/// GPIO9 should be driven high and low at a 1hz rate
///
unsigned char ReadPiHelloToggle(void)
{
   if((P1IN & PORT17_BITMASK_PI_HELLO_TOGGLE) != 0)
   {
      return 1;
   }
   else
   {
      return 0;
   }
}



///################################################
/// READ MAIN SUPPLY ALIVE
///
/// This function calls an AtoD read for the Main Supply input voltage.
/// If we see 400/1024 input on the AtoD then we know the power supply
/// has enough juice to run the Raspberry PI, so we report that the
/// main supply is alive.
///
unsigned char ReadMainSupplyAlive(void)
{
   unsigned short bar;
   bar = ADC_ReadUnbufferedAtoDPort(INCH_0, REF2_5V);
   if(bar > 400)
   {
      return TRUE;
   }
   else
   {
      return FALSE;
   }
}


///################################################
// READ GEL CELL HEALTHY
//
// This function is used to see if the GelCel is present.
// In the current version of the PWRMAN board this also
// reads true if the charger is connected.  Since the charger
// is always connected, you pretty much get a true if the
// main supply is hooked up, making this measurement useless.
// It's here in the code in case we have use for it later.
//
unsigned char ReadGelCellHealthy(void)
{
   unsigned short bar;
   bar = ADC_ReadUnbufferedAtoDPort(INCH_2, REF2_5V);
   if(bar > 400)
   {
      return TRUE;
   }
   else
   {
      return FALSE;
   }
}




///################################################
///
/// READ PUSHBUTTONS
///
/// Returns 0 if no button pressed, 1 if GREEN is pressed, or 2 if RED is pressed
///
/// The pushbutton switches are each hooked to a resistor which causes a different
/// analog voltage to show up at AtoD channel 1.  Here we decide which of the
/// two (if any) switch is pushed and return an appropriate constant.
///
unsigned char ReadPushButtons(void)
{
   unsigned short bar;
   bar = ADC_ReadUnbufferedAtoDPort(INCH_1, REF2_5V);
   if(bar < 300)
   {
      return(NOTHING_PUSHED);
   }
   else if(bar < 800)
   {
      return(GREEN_PUSHED);
   }
   return(RED_PUSHED);
}




///######################################################
// WRITE ALL LEDS
//
// This function is used by test mode to drive all 3 LEDs with one function call.
//
//void WriteAllLeds(unsigned char passed_blue, unsigned char passed_red, unsigned char passed_green)
//{
//   WriteBlueLED(passed_blue);
//   WriteRedLED(passed_red);
//   WriteGreenLED(passed_green);
//}



///######################################################
// SET BLUE LED
//
// passed parameter selects if we are to turn the LED on or off.   TRUE is turn-on

void WriteBlueLED(unsigned char passed_lightBlueBlinky)
{
   if(passed_lightBlueBlinky)
   {
      P1OUT &= ~PORT13_BITMASK_BLUE_LED;    //turn on BLUE switch LED by making the output low
   }
   else
   {
      P1OUT |= PORT13_BITMASK_BLUE_LED;    //turn off BLUE switch LED by making the output high
   }
}



///######################################################
// SET GREEN LED
//
// passed parameter selects if we are to turn the LED on or off.   TRUE is turn-on

void WriteGreenLED(unsigned char passed_lightGreenButton)
{
   if(passed_lightGreenButton)
   {
      P2OUT |= PORT27_BITMASK_GREEN_LED;      //turn on GREEN switch LED by making the output high
   }
   else
   {
      P2OUT &= ~PORT27_BITMASK_GREEN_LED;     //turn off GREEN switch LED by making the output low
   }
}



///######################################################
// SET RED LED
//
// passed parameter selects if we are to turn the LED on or off.   TRUE is turn-on

void WriteRedLED(unsigned char passed_lightRedButton)
{
   if(passed_lightRedButton)
   {
      P1OUT |= PORT14_BITMASK_RED_LED;     //turn on RED switch LED by making the output high
   }
   else
   {
      P1OUT &= ~PORT14_BITMASK_RED_LED;    //turn off RED switch LED by making the output low
   }
}

